@@ -70,7 +70,7 @@ from inspect import Parameter, signature
 import json
 import logging
 import os
-from typing import Any, Callable, Dict, List, Optional, Tuple
+from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 from colorlog import ColoredFormatter
 
@@ -93,12 +93,20 @@ LOG_COLORS = {'DEBUG': 'cyan',
               'CRITICAL': 'red,bg_white'}
 
 
-def get_colour_handler(extranames: List[str] = None) -> logging.StreamHandler:
+def get_colour_handler(extranames: List[str] = None,
+                       with_process_id: bool = False,
+                       with_thread_id: bool = False) -> logging.StreamHandler:
+    fmt = "%(cyan)s%(asctime)s.%(msecs)03d "
+    if with_process_id or with_thread_id:
+        procinfo = []  # type: List[str]
+        if with_process_id:
+            procinfo.append("p%(process)d")
+        if with_thread_id:
+            procinfo.append("t%(thread)d")
+        fmt += "[{}]".format(".".join(procinfo))
     extras = ":" + ":".join(extranames) if extranames else ""
-    fmt = (
-        "%(cyan)s%(asctime)s.%(msecs)03d %(name)s{extras}:%(levelname)s: "
-        "%(log_color)s%(message)s"
-    ).format(extras=extras)
+    fmt += " %(name)s{extras}:%(levelname)s: ".format(extras=extras)
+    fmt += "%(log_color)s%(message)s"
     cf = ColoredFormatter(fmt,
                           datefmt=LOG_DATEFMT,
                           reset=True,
@@ -113,7 +121,9 @@ def get_colour_handler(extranames: List[str] = None) -> logging.StreamHandler:
 def configure_logger_for_colour(logger: logging.Logger,
                                 level: int = logging.INFO,
                                 remove_existing: bool = False,
-                                extranames: List[str] = None) -> None:
+                                extranames: List[str] = None,
+                                with_process_id: bool = False,
+                                with_thread_id: bool = False) -> None:
     """
     Applies a preconfigured datetime/colour scheme to a logger.
     Should ONLY be called from the "if __name__ == 'main'" script:
@@ -121,15 +131,23 @@ def configure_logger_for_colour(logger: logging.Logger,
     """
     if remove_existing:
         logger.handlers = []  # http://stackoverflow.com/questions/7484454
-    logger.addHandler(get_colour_handler(extranames))
+    handler = get_colour_handler(extranames,
+                                 with_process_id=with_process_id,
+                                 with_thread_id=with_thread_id)
+    handler.setLevel(level)
+    logger.addHandler(handler)
     logger.setLevel(level)
 
 
-def main_only_quicksetup_rootlogger(level: int = logging.DEBUG) -> None:
+def main_only_quicksetup_rootlogger(level: int = logging.DEBUG,
+                                    with_process_id: bool = False,
+                                    with_thread_id: bool = False) -> None:
     # Nasty. Only call from "if __name__ == '__main__'" clauses!
     rootlogger = logging.getLogger()
-    configure_logger_for_colour(rootlogger, level)
-    logging.basicConfig(level=logging.DEBUG)
+    configure_logger_for_colour(rootlogger, level, remove_existing=True,
+                                with_process_id=with_process_id,
+                                with_thread_id=with_thread_id)
+    # logging.basicConfig(level=level)
 
 
 # =============================================================================
@@ -268,19 +286,27 @@ def get_handler_report(h: logging.Handler) -> Dict[str, Any]:
     }
 
 
-def get_log_report(log: logging.Logger) -> Dict[str, Any]:
+def get_log_report(log: Union[logging.Logger,
+                              logging.PlaceHolder]) -> Dict[str, Any]:
     """Returns information on a log, as a dictionary. For debugging."""
-    # suppress invalid error for Logger.manager:
-    # noinspection PyUnresolvedReferences
-    return {
-        '(object)': str(log),
-        'level': log.level,
-        'disabled': log.disabled,
-        'propagate': log.propagate,
-        'parent': str(log.parent),
-        'manager': str(log.manager),
-        'handlers': [get_handler_report(h) for h in log.handlers],
-    }
+    if isinstance(log, logging.Logger):
+        # suppress invalid error for Logger.manager:
+        # noinspection PyUnresolvedReferences
+        return {
+            '(object)': str(log),
+            'level': log.level,
+            'disabled': log.disabled,
+            'propagate': log.propagate,
+            'parent': str(log.parent),
+            'manager': str(log.manager),
+            'handlers': [get_handler_report(h) for h in log.handlers],
+        }
+    elif isinstance(log, logging.PlaceHolder):
+        return {
+            "(object)": str(log),
+        }
+    else:
+        raise ValueError("Unknown object type: {!r}".format(log))
 
 
 def print_report_on_all_logs() -> None:
@@ -291,10 +317,18 @@ def print_report_on_all_logs() -> None:
     # noinspection PyUnresolvedReferences
     for name, obj in logging.Logger.manager.loggerDict.items():
         d[name] = get_log_report(obj)
-    d['(root logger)'] = get_log_report(logging.getLogger())
+    rootlogger = logging.getLogger()
+    d['(root logger)'] = get_log_report(rootlogger)
     print(json.dumps(d, sort_keys=True, indent=4, separators=(',', ': ')))
 
 
+def set_level_for_logger_and_its_handlers(log: logging.Logger,
+                                          level: int) -> None:
+    log.setLevel(level)
+    for h in log.handlers:  # type: logging.Handler
+        h.setLevel(level)
+
+
 # =============================================================================
 # HTML formatter
 # =============================================================================
@@ -377,7 +411,7 @@ class HtmlColorHandler(logging.StreamHandler):
         try:
             html = self.format(record)
             self.logfunction(html)
-        except:
+        except:  # nopep8
             self.handleError(record)
 
 
